@@ -580,202 +580,208 @@ class GMR(GaussianMixture):
         params_df = pd.DataFrame(params_temp_list, columns=['number of components', 'covariance type', 'rep']) # 全組み合わせのDataframe
         params_df = pd.get_dummies(params_df, columns=['covariance type', 'rep']) # ダミー変数化
         
-        # ベイズ最適化の繰り返し
-        for bo_iter in bo_iterations:
-            if self.display_flag:
-                print(f'Bayesian optimization iteration : {bo_iter + 1} / {bo_iteration_number}')
-        #    print('='*10)
-            if bo_iter == 0: # 最初の試行ではD最適基準を計算
-                # D最適基準の計算
-                autoscaled_params_df = (params_df - params_df.mean(axis=0)) / params_df.std(axis=0, ddof=1) # 計算のために標準化
-        
-                all_indexes = list(range(autoscaled_params_df.shape[0])) # indexを取得
-        
-                np.random.seed(11) # 乱数を生成するためのシードを固定
-                for random_search_number in range(doe_number_of_random_searches):
-                    # 1. ランダムに候補を選択
-                    new_selected_indexes = np.random.choice(all_indexes, doe_number_of_selecting_samples, replace=False)
-                    new_selected_samples = autoscaled_params_df.iloc[new_selected_indexes, :]
-                    # 2. D 最適基準を計算
-                    xt_x = np.dot(new_selected_samples.T, new_selected_samples)
-                    d_optimal_value = np.linalg.det(xt_x) 
-                    # 3. D 最適基準が前回までの最大値を上回ったら、選択された候補を更新
-                    if random_search_number == 0:
-                        best_d_optimal_value = d_optimal_value.copy()
-                        selected_sample_indexes = new_selected_indexes.copy()
-                    else:
-                        if best_d_optimal_value < d_optimal_value:
+        if params_df.shape[0] <= doe_number_of_selecting_samples + bo_iteration_number:
+            self.cv_opt(dataset, numbers_of_input_variables, numbers_of_output_variables, covariance_types,
+                        numbers_of_components, fold_number)
+        else:
+            # ベイズ最適化の繰り返し
+            for bo_iter in bo_iterations:
+                if self.display_flag:
+                    print(f'Bayesian optimization iteration : {bo_iter + 1} / {bo_iteration_number}')
+            #    print('='*10)
+                if bo_iter == 0: # 最初の試行ではD最適基準を計算
+                    # D最適基準の計算
+                    autoscaled_params_df = (params_df - params_df.mean(axis=0)) / params_df.std(axis=0, ddof=1) # 計算のために標準化
+            
+                    all_indexes = list(range(autoscaled_params_df.shape[0])) # indexを取得
+            
+                    np.random.seed(11) # 乱数を生成するためのシードを固定
+                    for random_search_number in range(doe_number_of_random_searches):
+                        # 1. ランダムに候補を選択
+                        new_selected_indexes = np.random.choice(all_indexes, doe_number_of_selecting_samples, replace=False)
+                        new_selected_samples = autoscaled_params_df.iloc[new_selected_indexes, :]
+                        # 2. D 最適基準を計算
+                        xt_x = np.dot(new_selected_samples.T, new_selected_samples)
+                        d_optimal_value = np.linalg.det(xt_x) 
+                        # 3. D 最適基準が前回までの最大値を上回ったら、選択された候補を更新
+                        if random_search_number == 0:
                             best_d_optimal_value = d_optimal_value.copy()
                             selected_sample_indexes = new_selected_indexes.copy()
-                selected_sample_indexes = list(selected_sample_indexes) # リスト型に変換
-        
-                # 選択されたサンプル、選択されなかったサンプル
-                selected_params_df = params_df.iloc[selected_sample_indexes, :]  # 選択されたサンプル
-                bo_params_df = selected_params_df.copy() # BOのGPモデル構築用データを作成
-                remaining_indexes = np.delete(all_indexes, selected_sample_indexes)  # 選択されなかったサンプルのインデックス
-                remaining_params_df = params_df.iloc[remaining_indexes, :]  # 選択されなかったサンプル
-        
-                # 選択された全候補でGMRの計算
-                params_with_score_df = params_df.copy() # cvのscoreが含まれるdataframe
-                params_with_score_df['r2cv score'] = np.nan # 初期値はnanを設定
-        
-            else: # 2回目以降では前回の結果をもとにする
-                selected_sample_indexes = next_samples_df.index # 提案サンプルのindex
-                selected_params_df = params_df.loc[selected_sample_indexes, :] # 次に計算するサンプル
-                bo_params_df = pd.concat([bo_params_df, selected_params_df], axis=0) # BOのGPモデル構築用データは前回のデータと提案サンプルをマージする
-                remaining_params_df = params_df.loc[params_with_score_df['r2cv score'].isna(), :] # 選択されなかったサンプル
-                remaining_params_df = remaining_params_df.drop(index=selected_sample_indexes)
-        
-            # 選ばれたサンプル（パラメータの組み合わせ）を一つずつ計算する
-            for i_n, selected_params_idx in enumerate(selected_sample_indexes):
-                selected_params = selected_params_df.loc[selected_params_idx, :] # サンプルの選択
-                # componentの数の決定
-                number_of_components = selected_params['number of components']
-                # covariance typeの決定
-                covariance_series = selected_params[[x for x in selected_params.index if 'covariance type' in x]]
-                covariance_type = covariance_series.loc[covariance_series == 1]
-                covariance_type = (covariance_type.index[0]).replace('covariance type_', '')
-                # repの決定
-                rep_series = selected_params[[x for x in selected_params.index if 'rep' in x]]
-                rep_type = rep_series.loc[rep_series == 1]
-                rep_type = (rep_type.index[0]).replace('rep_', '')
-                # GMRモデルの構築
-                self.covariance_type = covariance_type
-                self.n_components = number_of_components
-                self.rep = rep_type
-                estimated_y_in_cv = np.empty((dataset.shape[0],  len(numbers_of_output_variables))) # yの保存先
-                for i, (cv_train_idx, cv_test_idx) in enumerate(gmr_kfold.split(dataset)): # CVによる検証
-                    # CVのinnerとouterを設定
-                    autoscaled_train_innercv = dataset[cv_train_idx, :]
-                    autoscaled_train_outercv = dataset[cv_test_idx, :]
-                    try:
-                        # modelにfitさせる
-                        self.fit(autoscaled_train_innercv)
-                        # 学習モデルにouterを入力
-                        predict_y_train_outercv = self.predict_rep(
-                                autoscaled_train_outercv[:, numbers_of_input_variables], numbers_of_input_variables, numbers_of_output_variables)
-                    except:
-                        predict_y_train_outercv = np.ones([autoscaled_train_outercv.shape[0], 1]) * (-10**10)
-                    estimated_y_in_cv[cv_test_idx, :] = predict_y_train_outercv
-                
-                # r2を計算
-                y_train_one = np.ravel(dataset[:, numbers_of_output_variables])
-                y_pred = np.ravel(estimated_y_in_cv)
-                gmr_r2_score = r2_score(y_train_one, y_pred)
-                params_with_score_df.loc[selected_params_idx, 'r2cv score'] = gmr_r2_score # データの保存
-            if self.display_flag:
-                print('Best r2cv :', params_with_score_df['r2cv score'].max())
-                print('='*10)
+                        else:
+                            if best_d_optimal_value < d_optimal_value:
+                                best_d_optimal_value = d_optimal_value.copy()
+                                selected_sample_indexes = new_selected_indexes.copy()
+                    selected_sample_indexes = list(selected_sample_indexes) # リスト型に変換
             
-            # 最後はBOの計算をしないためbreak
-            if bo_iter + 1 == bo_iteration_number:
-                break
+                    # 選択されたサンプル、選択されなかったサンプル
+                    selected_params_df = params_df.iloc[selected_sample_indexes, :]  # 選択されたサンプル
+                    bo_params_df = selected_params_df.copy() # BOのGPモデル構築用データを作成
+                    remaining_indexes = np.delete(all_indexes, selected_sample_indexes)  # 選択されなかったサンプルのインデックス
+                    remaining_params_df = params_df.iloc[remaining_indexes, :]  # 選択されなかったサンプル
+            
+                    # 選択された全候補でGMRの計算
+                    params_with_score_df = params_df.copy() # cvのscoreが含まれるdataframe
+                    params_with_score_df['r2cv score'] = np.nan # 初期値はnanを設定
+            
+                else: # 2回目以降では前回の結果をもとにする
+                    selected_sample_indexes = next_samples_df.index # 提案サンプルのindex
+                    selected_params_df = params_df.loc[selected_sample_indexes, :] # 次に計算するサンプル
+                    bo_params_df = pd.concat([bo_params_df, selected_params_df], axis=0) # BOのGPモデル構築用データは前回のデータと提案サンプルをマージする
+                    remaining_params_df = params_df.loc[params_with_score_df['r2cv score'].isna(), :] # 選択されなかったサンプル
+                    remaining_params_df = remaining_params_df.drop(index=selected_sample_indexes)
+            
+                # 選ばれたサンプル（パラメータの組み合わせ）を一つずつ計算する
+                for i_n, selected_params_idx in enumerate(selected_sample_indexes):
+                    selected_params = selected_params_df.loc[selected_params_idx, :] # サンプルの選択
+                    # componentの数の決定
+                    number_of_components = selected_params['number of components']
+                    # covariance typeの決定
+                    covariance_series = selected_params[[x for x in selected_params.index if 'covariance type' in x]]
+                    covariance_type = covariance_series.loc[covariance_series == 1]
+                    covariance_type = (covariance_type.index[0]).replace('covariance type_', '')
+                    # repの決定
+                    rep_series = selected_params[[x for x in selected_params.index if 'rep' in x]]
+                    rep_type = rep_series.loc[rep_series == 1]
+                    rep_type = (rep_type.index[0]).replace('rep_', '')
+                    # GMRモデルの構築
+                    self.covariance_type = covariance_type
+                    self.n_components = number_of_components
+                    self.rep = rep_type
+                    estimated_y_in_cv = np.empty((dataset.shape[0],  len(numbers_of_output_variables))) # yの保存先
+                    for i, (cv_train_idx, cv_test_idx) in enumerate(gmr_kfold.split(dataset)): # CVによる検証
+                        # CVのinnerとouterを設定
+                        autoscaled_train_innercv = dataset[cv_train_idx, :]
+                        autoscaled_train_outercv = dataset[cv_test_idx, :]
+                        try:
+                            # modelにfitさせる
+                            self.fit(autoscaled_train_innercv)
+                            # 学習モデルにouterを入力
+                            predict_y_train_outercv = self.predict_rep(
+                                    autoscaled_train_outercv[:, numbers_of_input_variables], numbers_of_input_variables, numbers_of_output_variables)
+                        except:
+                            predict_y_train_outercv = np.ones([autoscaled_train_outercv.shape[0], 1]) * (-10**10)
+                        estimated_y_in_cv[cv_test_idx, :] = predict_y_train_outercv
                     
-            # Bayesian optimization
-            bo_x_data = bo_params_df.copy() # GP学習用データはGMRの結果があるサンプル
-            bo_x_prediction = remaining_params_df.copy() # predictionは選択されていない（GMRの結果がない）サンプル
-            bo_y_data = params_with_score_df.loc[bo_params_df.index, 'r2cv score'] # yはGMRのr2cv
-            
-            # カーネル 11 種類
-            bo_kernels = [ConstantKernel() * DotProduct() + WhiteKernel(),
-                        ConstantKernel() * RBF() + WhiteKernel(),
-                        ConstantKernel() * RBF() + WhiteKernel() + ConstantKernel() * DotProduct(),
-                        ConstantKernel() * RBF(np.ones(bo_x_data.shape[1])) + WhiteKernel(),
-                        ConstantKernel() * RBF(np.ones(bo_x_data.shape[1])) + WhiteKernel() + ConstantKernel() * DotProduct(),
-                        ConstantKernel() * Matern(nu=1.5) + WhiteKernel(),
-                        ConstantKernel() * Matern(nu=1.5) + WhiteKernel() + ConstantKernel() * DotProduct(),
-                        ConstantKernel() * Matern(nu=0.5) + WhiteKernel(),
-                        ConstantKernel() * Matern(nu=0.5) + WhiteKernel() + ConstantKernel() * DotProduct(),
-                        ConstantKernel() * Matern(nu=2.5) + WhiteKernel(),
-                        ConstantKernel() * Matern(nu=2.5) + WhiteKernel() + ConstantKernel() * DotProduct()]
-        
-            next_samples = pd.DataFrame([], columns=selected_params_df.columns)  # 次のサンプルを入れる変数を準備
-        
-            # 次の候補を複数提案する繰り返し工程
-            for bo_sample_number in range(bo_number_of_selecting_samples):
-                # オートスケーリング
-                autoscaled_bo_y_data = (bo_y_data - bo_y_data.mean()) / bo_y_data.std()
-                autoscaled_bo_x_data = (bo_x_data - bo_x_data.mean()) / bo_x_data.std()
-                autoscaled_bo_x_prediction = (bo_x_prediction - bo_x_data.mean()) / bo_x_data.std()
+                    # r2を計算
+                    y_train_one = np.ravel(dataset[:, numbers_of_output_variables])
+                    y_pred = np.ravel(estimated_y_in_cv)
+                    gmr_r2_score = r2_score(y_train_one, y_pred)
+                    params_with_score_df.loc[selected_params_idx, 'r2cv score'] = gmr_r2_score # データの保存
+                if self.display_flag:
+                    print('Best r2cv :', params_with_score_df['r2cv score'].max())
+                    print('='*10)
                 
-                # モデル構築
-                if bo_regression_method == 'gpr_one_kernel':
-                    bo_selected_kernel = bo_kernels[bo_kernel_number]
-                    bo_model = GaussianProcessRegressor(alpha=0, kernel=bo_selected_kernel)
-        
-                elif bo_regression_method == 'gpr_kernels':
-                    # クロスバリデーションによるカーネル関数の最適化
-                    bo_cross_validation = KFold(n_splits=bo_gp_fold_number, random_state=9, shuffle=True) # クロスバリデーションの分割の設定
-                    bo_r2cvs = [] # 空の list。カーネル関数ごとに、クロスバリデーション後の r2 を入れていきます
-                    for index, bo_kernel in enumerate(bo_kernels):
-                        bo_model = GaussianProcessRegressor(alpha=0, kernel=bo_kernel)
-                        estimated_bo_y_in_cv = np.ndarray.flatten(cross_val_predict(bo_model, autoscaled_bo_x_data, autoscaled_bo_y_data, cv=bo_cross_validation))
-                        estimated_bo_y_in_cv = estimated_bo_y_in_cv * bo_y_data.std(ddof=1) + bo_y_data.mean()
-                        bo_r2cvs.append(r2_score(bo_y_data, estimated_bo_y_in_cv))
-                    optimal_bo_kernel_number = np.where(bo_r2cvs == np.max(bo_r2cvs))[0][0]  # クロスバリデーション後の r2 が最も大きいカーネル関数の番号
-                    optimal_bo_kernel = bo_kernels[optimal_bo_kernel_number]  # クロスバリデーション後の r2 が最も大きいカーネル関数
+                # 最後はBOの計算をしないためbreak
+                if bo_iter + 1 == bo_iteration_number:
+                    break
+                        
+                # Bayesian optimization
+                bo_x_data = bo_params_df.copy() # GP学習用データはGMRの結果があるサンプル
+                bo_x_prediction = remaining_params_df.copy() # predictionは選択されていない（GMRの結果がない）サンプル
+                bo_y_data = params_with_score_df.loc[bo_params_df.index, 'r2cv score'] # yはGMRのr2cv
+                
+                # カーネル 11 種類
+                bo_kernels = [ConstantKernel() * DotProduct() + WhiteKernel(),
+                            ConstantKernel() * RBF() + WhiteKernel(),
+                            ConstantKernel() * RBF() + WhiteKernel() + ConstantKernel() * DotProduct(),
+                            ConstantKernel() * RBF(np.ones(bo_x_data.shape[1])) + WhiteKernel(),
+                            ConstantKernel() * RBF(np.ones(bo_x_data.shape[1])) + WhiteKernel() + ConstantKernel() * DotProduct(),
+                            ConstantKernel() * Matern(nu=1.5) + WhiteKernel(),
+                            ConstantKernel() * Matern(nu=1.5) + WhiteKernel() + ConstantKernel() * DotProduct(),
+                            ConstantKernel() * Matern(nu=0.5) + WhiteKernel(),
+                            ConstantKernel() * Matern(nu=0.5) + WhiteKernel() + ConstantKernel() * DotProduct(),
+                            ConstantKernel() * Matern(nu=2.5) + WhiteKernel(),
+                            ConstantKernel() * Matern(nu=2.5) + WhiteKernel() + ConstantKernel() * DotProduct()]
+            
+                next_samples = pd.DataFrame([], columns=selected_params_df.columns)  # 次のサンプルを入れる変数を準備
+            
+                # 次の候補を複数提案する繰り返し工程
+                for bo_sample_number in range(bo_number_of_selecting_samples):
+                    # オートスケーリング
+                    bo_x_data_std = bo_x_data.std()
+                    bo_x_data_std[bo_x_data_std == 0] = 1
+                    autoscaled_bo_y_data = (bo_y_data - bo_y_data.mean()) / bo_y_data.std()
+                    autoscaled_bo_x_data = (bo_x_data - bo_x_data.mean()) / bo_x_data_std
+                    autoscaled_bo_x_prediction = (bo_x_prediction - bo_x_data.mean()) / bo_x_data_std
                     
                     # モデル構築
-                    bo_model = GaussianProcessRegressor(alpha=0, kernel=optimal_bo_kernel) # GPR モデルの宣言
+                    if bo_regression_method == 'gpr_one_kernel':
+                        bo_selected_kernel = bo_kernels[bo_kernel_number]
+                        bo_model = GaussianProcessRegressor(alpha=0, kernel=bo_selected_kernel)
+            
+                    elif bo_regression_method == 'gpr_kernels':
+                        # クロスバリデーションによるカーネル関数の最適化
+                        bo_cross_validation = KFold(n_splits=bo_gp_fold_number, random_state=9, shuffle=True) # クロスバリデーションの分割の設定
+                        bo_r2cvs = [] # 空の list。カーネル関数ごとに、クロスバリデーション後の r2 を入れていきます
+                        for index, bo_kernel in enumerate(bo_kernels):
+                            bo_model = GaussianProcessRegressor(alpha=0, kernel=bo_kernel)
+                            estimated_bo_y_in_cv = np.ndarray.flatten(cross_val_predict(bo_model, autoscaled_bo_x_data, autoscaled_bo_y_data, cv=bo_cross_validation))
+                            estimated_bo_y_in_cv = estimated_bo_y_in_cv * bo_y_data.std(ddof=1) + bo_y_data.mean()
+                            bo_r2cvs.append(r2_score(bo_y_data, estimated_bo_y_in_cv))
+                        optimal_bo_kernel_number = np.where(bo_r2cvs == np.max(bo_r2cvs))[0][0]  # クロスバリデーション後の r2 が最も大きいカーネル関数の番号
+                        optimal_bo_kernel = bo_kernels[optimal_bo_kernel_number]  # クロスバリデーション後の r2 が最も大きいカーネル関数
+                        
+                        # モデル構築
+                        bo_model = GaussianProcessRegressor(alpha=0, kernel=optimal_bo_kernel) # GPR モデルの宣言
+                        
+                    bo_model.fit(autoscaled_bo_x_data, autoscaled_bo_y_data)  # モデルの学習
                     
-                bo_model.fit(autoscaled_bo_x_data, autoscaled_bo_y_data)  # モデルの学習
-                
-                # 予測
-                estimated_bo_y_prediction, estimated_bo_y_prediction_std = bo_model.predict(autoscaled_bo_x_prediction, return_std=True)
-                estimated_bo_y_prediction = estimated_bo_y_prediction * bo_y_data.std() + bo_y_data.mean()
-                estimated_bo_y_prediction_std = estimated_bo_y_prediction_std * bo_y_data.std()
-                
-                cumulative_variance = np.zeros(bo_x_prediction.shape[0])
-                # 獲得関数の計算
-                if acquisition_function == 'MI':
-                    acquisition_function_prediction = estimated_bo_y_prediction + np.log(2 / delta) ** 0.5 * (
-                            (estimated_bo_y_prediction_std ** 2 + cumulative_variance) ** 0.5 - cumulative_variance ** 0.5)
-                    cumulative_variance = cumulative_variance + estimated_bo_y_prediction_std ** 2
-                elif acquisition_function == 'EI':
-                    acquisition_function_prediction = (estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) * \
-                                                    norm.cdf((estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) /
-                                                                estimated_bo_y_prediction_std) + \
-                                                    estimated_bo_y_prediction_std * \
-                                                    norm.pdf((estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) /
-                                                                estimated_bo_y_prediction_std)
-                elif acquisition_function == 'PI':
-                    acquisition_function_prediction = norm.cdf(
-                            (estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) / estimated_bo_y_prediction_std)
-                elif acquisition_function == 'PTR':
-                    acquisition_function_prediction = norm.cdf(target_range[1],
-                                                            loc=estimated_bo_y_prediction,
-                                                            scale=estimated_bo_y_prediction_std
-                                                            ) - norm.cdf(target_range[0],
-                                                                            loc=estimated_bo_y_prediction,
-                                                                            scale=estimated_bo_y_prediction_std)
-                acquisition_function_prediction[estimated_bo_y_prediction_std <= 0] = 0
-                
-                # 保存
-                estimated_bo_y_prediction = pd.DataFrame(estimated_bo_y_prediction, bo_x_prediction.index, columns=['estimated_y'])
-                estimated_bo_y_prediction_std = pd.DataFrame(estimated_bo_y_prediction_std, bo_x_prediction.index, columns=['std_of_estimated_y'])
-                acquisition_function_prediction = pd.DataFrame(acquisition_function_prediction, index=bo_x_prediction.index, columns=['acquisition_function'])
-        #        
-                # 次のサンプル
-                next_samples = pd.concat([next_samples, bo_x_prediction.loc[acquisition_function_prediction.idxmax()]], axis=0)
-                
-                # x, y, x_prediction, cumulative_variance の更新
-                bo_x_data = pd.concat([bo_x_data, bo_x_prediction.loc[acquisition_function_prediction.idxmax()]], axis=0)
-                bo_y_data = pd.concat([bo_y_data, estimated_bo_y_prediction.loc[acquisition_function_prediction.idxmax()].iloc[0]], axis=0)
-                bo_x_prediction = bo_x_prediction.drop(acquisition_function_prediction.idxmax(), axis=0)
-                cumulative_variance = np.delete(cumulative_variance, np.where(acquisition_function_prediction.index == acquisition_function_prediction.iloc[:, 0].idxmax())[0][0])
-            next_samples_df = next_samples.copy()
-        
-        # 結果の保存
-        #params_with_score_df.sort_values('r2cv score', ascending=False).to_csv('params_with_score.csv')
-        params_with_score_df_best = params_with_score_df.sort_values('r2cv score', ascending=False).iloc[0, :] # r2が高い順にソー
-        # covariance typeの決定
-        covariance_series = params_with_score_df_best[[x for x in params_with_score_df_best.index if 'covariance type' in x]]
-        covariance_type = covariance_series.loc[covariance_series == 1]
-        # repの決定
-        rep_series = params_with_score_df_best[[x for x in params_with_score_df_best.index if 'rep' in x]]
-        rep_type = rep_series.loc[rep_series == 1]
-        self.covariance_type = (covariance_type.index[0]).replace('covariance type_', '')
-        self.n_components = int(params_with_score_df_best['number of components'])
-        self.rep = (rep_type.index[0]).replace('rep_', '')
-        self.r2cv = params_with_score_df_best['r2cv score']
+                    # 予測
+                    estimated_bo_y_prediction, estimated_bo_y_prediction_std = bo_model.predict(autoscaled_bo_x_prediction, return_std=True)
+                    estimated_bo_y_prediction = estimated_bo_y_prediction * bo_y_data.std() + bo_y_data.mean()
+                    estimated_bo_y_prediction_std = estimated_bo_y_prediction_std * bo_y_data.std()
+                    
+                    cumulative_variance = np.zeros(bo_x_prediction.shape[0])
+                    # 獲得関数の計算
+                    if acquisition_function == 'MI':
+                        acquisition_function_prediction = estimated_bo_y_prediction + np.log(2 / delta) ** 0.5 * (
+                                (estimated_bo_y_prediction_std ** 2 + cumulative_variance) ** 0.5 - cumulative_variance ** 0.5)
+                        cumulative_variance = cumulative_variance + estimated_bo_y_prediction_std ** 2
+                    elif acquisition_function == 'EI':
+                        acquisition_function_prediction = (estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) * \
+                                                        norm.cdf((estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) /
+                                                                    estimated_bo_y_prediction_std) + \
+                                                        estimated_bo_y_prediction_std * \
+                                                        norm.pdf((estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) /
+                                                                    estimated_bo_y_prediction_std)
+                    elif acquisition_function == 'PI':
+                        acquisition_function_prediction = norm.cdf(
+                                (estimated_bo_y_prediction - max(bo_y_data) - relaxation * bo_y_data.std()) / estimated_bo_y_prediction_std)
+                    elif acquisition_function == 'PTR':
+                        acquisition_function_prediction = norm.cdf(target_range[1],
+                                                                loc=estimated_bo_y_prediction,
+                                                                scale=estimated_bo_y_prediction_std
+                                                                ) - norm.cdf(target_range[0],
+                                                                                loc=estimated_bo_y_prediction,
+                                                                                scale=estimated_bo_y_prediction_std)
+                    acquisition_function_prediction[estimated_bo_y_prediction_std <= 0] = 0
+                    
+                    # 保存
+                    estimated_bo_y_prediction = pd.DataFrame(estimated_bo_y_prediction, bo_x_prediction.index, columns=['estimated_y'])
+                    estimated_bo_y_prediction_std = pd.DataFrame(estimated_bo_y_prediction_std, bo_x_prediction.index, columns=['std_of_estimated_y'])
+                    acquisition_function_prediction = pd.DataFrame(acquisition_function_prediction, index=bo_x_prediction.index, columns=['acquisition_function'])
+            #        
+                    # 次のサンプル
+                    next_samples = pd.concat([next_samples, bo_x_prediction.loc[acquisition_function_prediction.idxmax()]], axis=0)
+                    
+                    # x, y, x_prediction, cumulative_variance の更新
+                    bo_x_data = pd.concat([bo_x_data, bo_x_prediction.loc[acquisition_function_prediction.idxmax()]], axis=0)
+                    bo_y_data = pd.concat([bo_y_data, estimated_bo_y_prediction.loc[acquisition_function_prediction.idxmax()].iloc[0]], axis=0)
+                    bo_x_prediction = bo_x_prediction.drop(acquisition_function_prediction.idxmax(), axis=0)
+                    cumulative_variance = np.delete(cumulative_variance, np.where(acquisition_function_prediction.index == acquisition_function_prediction.iloc[:, 0].idxmax())[0][0])
+                next_samples_df = next_samples.copy()
+            
+            # 結果の保存
+            #params_with_score_df.sort_values('r2cv score', ascending=False).to_csv('params_with_score.csv')
+            params_with_score_df_best = params_with_score_df.sort_values('r2cv score', ascending=False).iloc[0, :] # r2が高い順にソー
+            # covariance typeの決定
+            covariance_series = params_with_score_df_best[[x for x in params_with_score_df_best.index if 'covariance type' in x]]
+            covariance_type = covariance_series.loc[covariance_series == 1]
+            # repの決定
+            rep_series = params_with_score_df_best[[x for x in params_with_score_df_best.index if 'rep' in x]]
+            rep_type = rep_series.loc[rep_series == 1]
+            self.covariance_type = (covariance_type.index[0]).replace('covariance type_', '')
+            self.n_components = int(params_with_score_df_best['number of components'])
+            self.rep = (rep_type.index[0]).replace('rep_', '')
+            self.r2cv = params_with_score_df_best['r2cv score']
         
\ No newline at end of file
