@@ -7,3 +7,5 @@
 from .basic_variable_selection import search_high_rate_of_same_values
 from .basic_variable_selection import search_highly_correlated_variables
 from .basic_variable_selection import clustering_based_on_correlation_coefficients
+from .variable_importance import cvpfi
+from .variable_importance import cvpfi_gmr
